@@ -38,8 +38,46 @@ get_tree <- function(forest, index) {
     stop(paste("The provided index,", index, "is not valid."))
   }
 
-  tree <- deserialize_tree(forest, index)
-  class(tree) <- "grf_tree"
+  # Convert internal grf representation to adjacency list.
+  # +1 from C++ to R index.
+  root <- forest[["_root_nodes"]][[index]] + 1
+  left <- forest[["_child_nodes"]][[index]][[1]]
+  right <- forest[["_child_nodes"]][[index]][[2]]
+  split_vars <- forest[["_split_vars"]][[index]]
+  split_values <- forest[["_split_values"]][[index]]
+  leaf_samples <- forest[["_leaf_samples"]][[index]]
+  drawn_samples <- forest[["_drawn_samples"]][[index]] + 1
+
+  nodes <- list()
+  frontier <- root
+  i <- 0
+  node.index <- 1
+  while (length(frontier) > 0) {
+    node <- frontier[1]
+    frontier <- frontier[-1]
+    i <- i + 1
+    if (left[[node]] == 0 && right[[node]] == 0) {
+      nodes[[i]] <- list(
+        is_leaf = TRUE,
+        samples = leaf_samples[[node]] + 1
+      )
+    } else {
+      nodes[[i]] <- list(
+        is_leaf = FALSE,
+        split_variable = split_vars[node] + 1,
+        split_value = split_values[node],
+        left_child = node.index + 1,
+        right_child = node.index + 2
+      )
+      node.index <- node.index + 2
+      frontier <- c(left[node] + 1, right[node] + 1, frontier)
+    }
+  }
+
+  tree <- list()
+  tree$num_samples <- length(drawn_samples)
+  tree$drawn_samples <- drawn_samples
+  tree$nodes <- nodes
 
   columns <- colnames(forest$X.orig)
   indices <- 1:ncol(forest$X.orig)
@@ -59,7 +97,7 @@ get_tree <- function(forest, index) {
     node
   })
 
-
+  class(tree) <- "grf_tree"
   tree
 }
 
@@ -93,6 +131,8 @@ split_frequencies <- function(forest, max.depth = 4) {
 
 #' Calculate a simple measure of 'importance' for each feature.
 #'
+#' A simple weighted sum of how many times feature i was split on at each depth in the forest.
+#'
 #' @param forest The trained forest.
 #' @param decay.exponent A tuning parameter that controls the importance of split depth.
 #' @param max.depth Maximum depth of splits to consider.
@@ -160,11 +200,11 @@ get_sample_weights <- function(forest, newdata = NULL, num.threads = NULL) {
   if (!is.null(newdata)) {
     data <- create_data_matrices(newdata)
     compute_weights(
-      forest.short, train.data$default, train.data$sparse,
-      data$default, data$sparse, num.threads
+      forest.short, train.data$train.matrix, train.data$sparse.train.matrix,
+      data$train.matrix, data$sparse.train.matrix, num.threads
     )
   } else {
-    compute_weights_oob(forest.short, train.data$default, train.data$sparse, num.threads)
+    compute_weights_oob(forest.short, train.data$train.matrix, train.data$sparse.train.matrix, num.threads)
   }
 }
 
