@@ -16,26 +16,28 @@
 #'                      with size smaller than min.node.size can occur, as in the original randomForest package.
 #'                      Default is 5.
 #' @param honesty Whether to use honest splitting (i.e., sub-sample splitting). Default is TRUE.
+#'  For a detailed description of honesty, honesty.fraction, honesty.prune.leaves, and recommendations for
+#'  parameter tuning, see the grf
+#'  \href{https://grf-labs.github.io/grf/REFERENCE.html#honesty-honesty-fraction-honesty-prune-leaves}{algorithm reference}.
 #' @param honesty.fraction The fraction of data that will be used for determining splits if honesty = TRUE. Corresponds
-#'                         to set J1 in the notation of the paper. When using the defaults (honesty = TRUE and
-#'                         honesty.fraction = NULL), half of the data will be used for determining splits.
-#'                         Default is 0.5.
-#' @param prune.empty.leaves (experimental) If true, prunes the estimation sample tree such that no leaves
-#'  are empty. If false, keep the same tree as determined in the splits sample (if an empty leave is encountered, that
-#'  tree is skipped and does not contribute to the estimate). Setting this to false may improve performance on
-#'  small/marginally powered data, but requires more trees. Only applies if honesty is enabled. Default is TRUE.
+#'                         to set J1 in the notation of the paper. Default is 0.5 (i.e. half of the data is used for
+#'                         determining splits).
+#' @param honesty.prune.leaves If TRUE, prunes the estimation sample tree such that no leaves
+#'  are empty. If FALSE, keep the same tree as determined in the splits sample (if an empty leave is encountered, that
+#'  tree is skipped and does not contribute to the estimate). Setting this to FALSE may improve performance on
+#'  small/marginally powered data, but requires more trees (note: tuning does not adjust the number of trees).
+#'  Only applies if honesty is enabled. Default is TRUE.
 #' @param alpha A tuning parameter that controls the maximum imbalance of a split. Default is 0.05.
 #' @param imbalance.penalty A tuning parameter that controls how harshly imbalanced splits are penalized. Default is 0.
 #' @param clusters Vector of integers or factors specifying which cluster each observation corresponds to.
-#'                 Default is NULL (ignored).
-#' @param samples.per.cluster If sampling by cluster, the number of observations to be sampled from
-#'                            each cluster when training a tree. If NULL, we set samples.per.cluster to the size
-#'                            of the smallest cluster. If some clusters are smaller than samples.per.cluster,
-#'                            the whole cluster is used every time the cluster is drawn. Note that
-#'                            clusters with less than samples.per.cluster observations get relatively
-#'                            smaller weight than others in training the forest, i.e., the contribution
-#'                            of a given cluster to the final forest scales with the minimum of
-#'                            the number of observations in the cluster and samples.per.cluster. Default is NULL.
+#'  Default is NULL (ignored).
+#' @param equalize.cluster.weights If FALSE, each unit is given the same weight (so that bigger
+#'  clusters get more weight). If TRUE, each cluster is given equal weight in the forest. In this case,
+#'  during training, each tree uses the same number of observations from each drawn cluster: If the
+#'  smallest cluster has K units, then when we sample a cluster during training, we only give a random
+#'  K elements of the cluster to the tree-growing procedure. When estimating average treatment effects,
+#'  each observation is given weight 1/cluster size, so that the total weight of each cluster is the
+#'  same.
 #' @param compute.oob.predictions Whether OOB predictions on training set should be precomputed. Default is TRUE.
 #' @param num.threads Number of threads used in training. By default, the number of threads is set
 #'                    to the maximum hardware concurrency
@@ -61,41 +63,33 @@
 #' @export
 custom_forest <- function(X, Y,
                           sample.fraction = 0.5,
-                          mtry = NULL,
+                          mtry = min(ceiling(sqrt(ncol(X)) + 20), ncol(X)),
                           num.trees = 2000,
-                          min.node.size = NULL,
+                          min.node.size = 5,
                           honesty = TRUE,
-                          honesty.fraction = NULL,
-                          prune.empty.leaves = NULL,
+                          honesty.fraction = 0.5,
+                          honesty.prune.leaves = TRUE,
                           alpha = 0.05,
                           imbalance.penalty = 0.0,
                           clusters = NULL,
-                          samples.per.cluster = NULL,
+                          equalize.cluster.weights = FALSE,
                           compute.oob.predictions = TRUE,
                           num.threads = NULL,
-                          seed = NULL) {
+                          seed = runif(1, 0, .Machine$integer.max)) {
   validate_X(X)
   Y <- validate_observations(Y, X)
-
-  mtry <- validate_mtry(mtry, X)
-  num.threads <- validate_num_threads(num.threads)
-  min.node.size <- validate_min_node_size(min.node.size)
-  sample.fraction <- validate_sample_fraction(sample.fraction)
-  seed <- validate_seed(seed)
   clusters <- validate_clusters(clusters, X)
-  samples.per.cluster <- validate_samples_per_cluster(samples.per.cluster, clusters)
-  honesty.fraction <- validate_honesty_fraction(honesty.fraction, honesty)
-  prune.empty.leaves <- validate_prune_empty_leaves(prune.empty.leaves)
+  samples.per.cluster <- validate_equalize_cluster_weights(equalize.cluster.weights, clusters, NULL)
+  num.threads <- validate_num_threads(num.threads)
 
   no.split.variables <- numeric(0)
 
-  data <- create_data_matrices(X, Y)
-  outcome.index <- ncol(X) + 1
+  data <- create_data_matrices(X, outcome = Y)
   ci.group.size <- 1
 
   forest <- custom_train(
-    data$default, data$sparse, outcome.index, mtry, num.trees, min.node.size,
-    sample.fraction, honesty, honesty.fraction, prune.empty.leaves, ci.group.size, alpha,
+    data$train.matrix, data$sparse.train.matrix, data$outcome.index, mtry, num.trees, min.node.size,
+    sample.fraction, honesty, honesty.fraction, honesty.prune.leaves, ci.group.size, alpha,
     imbalance.penalty, clusters, samples.per.cluster, num.threads, compute.oob.predictions, seed
   )
 
@@ -141,7 +135,6 @@ predict.custom_forest <- function(object, newdata = NULL, num.threads = NULL, ..
 
   X <- object[["X.orig"]]
   train.data <- create_data_matrices(X, object[["Y.orig"]])
-  outcome.index <- ncol(X) + 1
 
   num.threads <- validate_num_threads(num.threads)
 
@@ -149,10 +142,10 @@ predict.custom_forest <- function(object, newdata = NULL, num.threads = NULL, ..
     validate_newdata(newdata, X)
     data <- create_data_matrices(newdata)
     custom_predict(
-      forest.short, train.data$default, train.data$sparse, outcome.index,
-      data$default, data$sparse, num.threads
+      forest.short, train.data$train.matrix, train.data$sparse.train.matrix, train.data$outcome.index,
+      data$train.matrix, data$sparse.train.matrix, num.threads
     )
   } else {
-    custom_predict_oob(forest.short, train.data$default, train.data$sparse, outcome.index, num.threads)
+    custom_predict_oob(forest.short, train.data$train.matrix, train.data$sparse.train.matrix, train.data$outcome.index, num.threads)
   }
 }
