@@ -5,43 +5,45 @@
 #'
 #' @param X The covariates used in the quantile regression.
 #' @param Y The outcome.
+#' @param num.trees Number of trees grown in the forest. Note: Getting accurate
+#'                  confidence intervals generally requires more trees than
+#'                  getting accurate predictions. Default is 2000.
 #' @param quantiles Vector of quantiles used to calibrate the forest. Default is (0.1, 0.5, 0.9).
 #' @param regression.splitting Whether to use regression splits when growing trees instead
 #'                             of specialized splits based on the quantiles (the default).
 #'                             Setting this flag to true corresponds to the approach to
 #'                             quantile forests from Meinshausen (2006). Default is FALSE.
+#' @param clusters Vector of integers or factors specifying which cluster each observation corresponds to.
+#'  Default is NULL (ignored).
+#' @param equalize.cluster.weights If FALSE, each unit is given the same weight (so that bigger
+#'  clusters get more weight). If TRUE, each cluster is given equal weight in the forest. In this case,
+#'  during training, each tree uses the same number of observations from each drawn cluster: If the
+#'  smallest cluster has K units, then when we sample a cluster during training, we only give a random
+#'  K elements of the cluster to the tree-growing procedure. When estimating average treatment effects,
+#'  each observation is given weight 1/cluster size, so that the total weight of each cluster is the
+#'  same.
 #' @param sample.fraction Fraction of the data used to build each tree.
 #'                        Note: If honesty = TRUE, these subsamples will
 #'                        further be cut by a factor of honesty.fraction. Default is 0.5.
 #' @param mtry Number of variables tried for each split. Default is
 #'             \eqn{\sqrt p + 20} where p is the number of variables.
-#' @param num.trees Number of trees grown in the forest. Note: Getting accurate
-#'                  confidence intervals generally requires more trees than
-#'                  getting accurate predictions. Default is 2000.
 #' @param min.node.size A target for the minimum number of observations in each tree leaf. Note that nodes
 #'                      with size smaller than min.node.size can occur, as in the original randomForest package.
 #'                      Default is 5.
 #' @param honesty Whether to use honest splitting (i.e., sub-sample splitting). Default is TRUE.
+#'  For a detailed description of honesty, honesty.fraction, honesty.prune.leaves, and recommendations for
+#'  parameter tuning, see the grf
+#'  \href{https://grf-labs.github.io/grf/REFERENCE.html#honesty-honesty-fraction-honesty-prune-leaves}{algorithm reference}.
 #' @param honesty.fraction The fraction of data that will be used for determining splits if honesty = TRUE. Corresponds
-#'                         to set J1 in the notation of the paper. When using the defaults (honesty = TRUE and
-#'                         honesty.fraction = NULL), half of the data will be used for determining splits.
-#'                         Default is 0.5.
-#' @param prune.empty.leaves (experimental) If true, prunes the estimation sample tree such that no leaves
-#'  are empty. If false, keep the same tree as determined in the splits sample (if an empty leave is encountered, that
-#'  tree is skipped and does not contribute to the estimate). Setting this to false may improve performance on
-#'  small/marginally powered data, but requires more trees. Only applies if honesty is enabled. Default is TRUE.
+#'                         to set J1 in the notation of the paper. Default is 0.5 (i.e. half of the data is used for
+#'                         determining splits).
+#' @param honesty.prune.leaves If TRUE, prunes the estimation sample tree such that no leaves
+#'  are empty. If FALSE, keep the same tree as determined in the splits sample (if an empty leave is encountered, that
+#'  tree is skipped and does not contribute to the estimate). Setting this to FALSE may improve performance on
+#'  small/marginally powered data, but requires more trees (note: tuning does not adjust the number of trees).
+#'  Only applies if honesty is enabled. Default is TRUE.
 #' @param alpha A tuning parameter that controls the maximum imbalance of a split. Default is 0.05.
 #' @param imbalance.penalty A tuning parameter that controls how harshly imbalanced splits are penalized. Default is 0.
-#' @param clusters Vector of integers or factors specifying which cluster each observation corresponds to.
-#'                 Default is NULL (ignored).
-#' @param samples.per.cluster If sampling by cluster, the number of observations to be sampled from
-#'                            each cluster when training a tree. If NULL, we set samples.per.cluster to the size
-#'                            of the smallest cluster. If some clusters are smaller than samples.per.cluster,
-#'                            the whole cluster is used every time the cluster is drawn. Note that
-#'                            clusters with less than samples.per.cluster observations get relatively
-#'                            smaller weight than others in training the forest, i.e., the contribution
-#'                            of a given cluster to the final forest scales with the minimum of
-#'                            the number of observations in the cluster and samples.per.cluster. Default is NULL.
 #' @param num.threads Number of threads used in training. By default, the number of threads is set
 #'                    to the maximum hardware concurrency.
 #' @param seed The seed of the C++ random number generator.
@@ -77,21 +79,21 @@
 #'
 #' @export
 quantile_forest <- function(X, Y,
+                            num.trees = 2000,
                             quantiles = c(0.1, 0.5, 0.9),
                             regression.splitting = FALSE,
+                            clusters = NULL,
+                            equalize.cluster.weights = FALSE,
                             sample.fraction = 0.5,
-                            mtry = NULL,
-                            num.trees = 2000,
-                            min.node.size = NULL,
+                            mtry = min(ceiling(sqrt(ncol(X)) + 20), ncol(X)),
+                            min.node.size = 5,
                             honesty = TRUE,
-                            honesty.fraction = NULL,
-                            prune.empty.leaves = NULL,
+                            honesty.fraction = 0.5,
+                            honesty.prune.leaves = TRUE,
                             alpha = 0.05,
                             imbalance.penalty = 0.0,
-                            clusters = NULL,
-                            samples.per.cluster = NULL,
                             num.threads = NULL,
-                            seed = NULL) {
+                            seed = runif(1, 0, .Machine$integer.max)) {
   if (!is.numeric(quantiles) | length(quantiles) < 1) {
     stop("Error: Must provide numeric quantiles")
   } else if (min(quantiles) <= 0 | max(quantiles) >= 1) {
@@ -100,25 +102,16 @@ quantile_forest <- function(X, Y,
 
   validate_X(X)
   Y <- validate_observations(Y, X)
-
-  mtry <- validate_mtry(mtry, X)
-  num.threads <- validate_num_threads(num.threads)
-  min.node.size <- validate_min_node_size(min.node.size)
-  sample.fraction <- validate_sample_fraction(sample.fraction)
-  seed <- validate_seed(seed)
   clusters <- validate_clusters(clusters, X)
-  samples.per.cluster <- validate_samples_per_cluster(samples.per.cluster, clusters)
-  honesty.fraction <- validate_honesty_fraction(honesty.fraction, honesty)
-  prune.empty.leaves <- validate_prune_empty_leaves(prune.empty.leaves)
-
-  data <- create_data_matrices(X, Y)
-  outcome.index <- ncol(X) + 1
+  samples.per.cluster <- validate_equalize_cluster_weights(equalize.cluster.weights, clusters, NULL)
+  num.threads <- validate_num_threads(num.threads)
 
+  data <- create_data_matrices(X, outcome = Y)
   ci.group.size <- 1
 
   forest <- quantile_train(
-    quantiles, regression.splitting, data$default, data$sparse, outcome.index, mtry,
-    num.trees, min.node.size, sample.fraction, honesty, honesty.fraction, prune.empty.leaves,
+    quantiles, regression.splitting, data$train.matrix, data$sparse.train.matrix, data$outcome.index, mtry,
+    num.trees, min.node.size, sample.fraction, honesty, honesty.fraction, honesty.prune.leaves,
     ci.group.size, alpha, imbalance.penalty, clusters, samples.per.cluster, num.threads, seed
   )
 
@@ -126,6 +119,7 @@ quantile_forest <- function(X, Y,
   forest[["X.orig"]] <- X
   forest[["Y.orig"]] <- Y
   forest[["clusters"]] <- clusters
+  forest[["equalize.cluster.weights"]] <- equalize.cluster.weights
   forest
 }
 
@@ -181,20 +175,19 @@ predict.quantile_forest <- function(object,
   forest.short <- object[-which(names(object) == "X.orig")]
 
   X <- object[["X.orig"]]
-  train.data <- create_data_matrices(X, object[["Y.orig"]])
-  outcome.index <- ncol(X) + 1
+  train.data <- create_data_matrices(X, outcome = object[["Y.orig"]])
 
   if (!is.null(newdata)) {
     validate_newdata(newdata, object$X.orig)
     data <- create_data_matrices(newdata)
     quantile_predict(
-      forest.short, quantiles, train.data$default, train.data$sparse, outcome.index,
-      data$default, data$sparse, num.threads
+      forest.short, quantiles, train.data$train.matrix, train.data$sparse.train.matrix,
+      train.data$outcome.index, data$train.matrix, data$sparse.train.matrix, num.threads
     )
   } else {
     quantile_predict_oob(
-      forest.short, quantiles, train.data$default, train.data$sparse,
-      outcome.index, num.threads
+      forest.short, quantiles, train.data$train.matrix, train.data$sparse.train.matrix,
+      train.data$outcome.index, num.threads
     )
   }
 }
