@@ -24,27 +24,43 @@
 #include "tree/TreeTrainer.h"
 #include "tree/Tree.h"
 
+namespace grf {
+
 class Forest {
 public:
-  static Forest create(std::vector<std::shared_ptr<Tree>> trees,
-                       const ForestOptions& forest_options,
-                       const Data* data);
-
-  Forest(const std::vector<std::shared_ptr<Tree>>& trees,
+  Forest(std::vector<std::unique_ptr<Tree>>& trees,
          size_t num_variables,
          size_t ci_group_size);
 
-  const std::vector<std::shared_ptr<Tree>>& get_trees() const;
+  Forest(Forest&& forest);
+
+  const std::vector<std::unique_ptr<Tree>>& get_trees() const;
+
+  /**
+   * A method intended for internal use that allows the list of
+   * trees to be modified.
+   */
+  std::vector<std::unique_ptr<Tree>>& get_trees_();
 
   const size_t get_num_variables() const;
   const size_t get_ci_group_size() const;
 
-  static Forest merge(const std::vector<std::shared_ptr<Forest>>& forests);
+  /**
+   * Merges the given forests into a single forest. The new forest
+   * will contain all the trees from the smaller forests.
+   *
+   * NOTE: this is a destructive operation -- the original forests cannot
+   * be used after they are merged together.
+   */
+  static Forest merge(std::vector<Forest>& forests);
   
 private:
-  std::vector<std::shared_ptr<Tree>> trees;
+  std::vector<std::unique_ptr<Tree>> trees;
   size_t num_variables;
   size_t ci_group_size;
+  DISALLOW_COPY_AND_ASSIGN(Forest);
 };
 
+} // namespace grf
+
 #endif /* GRF_FOREST_H_ */
