@@ -14,8 +14,11 @@
   You should have received a copy of the GNU General Public License
   along with grf. If not, see <http://www.gnu.org/licenses/>.
  #-------------------------------------------------------------------------------*/
+ 
 #include <thread>
 #include <random>
+#include <stdexcept>
+
 #include "forest/ForestOptions.h"
 #include "tree/TreeOptions.h"
 
@@ -28,7 +31,7 @@ ForestOptions::ForestOptions(uint num_trees,
                              uint min_node_size,
                              bool honesty,
                              double honesty_fraction,
-                             bool prune_empty_leaves,
+                             bool honesty_prune_leaves,
                              double alpha,
                              double imbalance_penalty,
                              uint num_threads,
@@ -37,7 +40,7 @@ ForestOptions::ForestOptions(uint num_trees,
                              uint samples_per_cluster):
     ci_group_size(ci_group_size),
     sample_fraction(sample_fraction),
-    tree_options(mtry, min_node_size, honesty, honesty_fraction, prune_empty_leaves, alpha, imbalance_penalty),
+    tree_options(mtry, min_node_size, honesty, honesty_fraction, honesty_prune_leaves, alpha, imbalance_penalty),
     sampling_options(samples_per_cluster, sample_clusters) {
 
   this->num_threads = validate_num_threads(num_threads);
