@@ -17,8 +17,13 @@
 
 #include "CustomRelabelingStrategy.h"
 
-std::unordered_map<size_t, double> CustomRelabelingStrategy::relabel(
+namespace grf {
+
+bool CustomRelabelingStrategy::relabel(
     const std::vector<size_t>& samples,
-    const Data* data) {
-  return std::unordered_map<size_t, double>();
+    const Data& data,
+    std::vector<double>& responses_by_sample) const {
+  return true;
 }
+
+} // namespace grf
