@@ -21,12 +21,16 @@
 
 #include "RelabelingStrategy.h"
 
-class CustomRelabelingStrategy: public RelabelingStrategy {
+namespace grf {
+
+class CustomRelabelingStrategy final: public RelabelingStrategy {
 public:
-  std::unordered_map<size_t, double> relabel(
+  bool relabel(
       const std::vector<size_t>& samples,
-      const Data* data);
+      const Data& data,
+      std::vector<double>& responses_by_sample) const;
 };
 
+} // namespace grf
 
 #endif //GRF_CUSTOMRELABELINGSTRATEGY_H
