@@ -18,21 +18,24 @@
 #ifndef GRF_INSTRUMENTALRELABELINGSTRATEGY_H
 #define GRF_INSTRUMENTALRELABELINGSTRATEGY_H
 
-#include <unordered_map>
 #include <vector>
-#include "commons/Data.h"
-#include "tree/Tree.h"
-#include "relabeling/RelabelingStrategy.h"
 
-class InstrumentalRelabelingStrategy: public RelabelingStrategy {
+#include "commons/Data.h"
+#include "relabeling/RelabelingStrategy.h"
+#include "tree/Tree.h"
+
+namespace grf {
+
+class InstrumentalRelabelingStrategy final: public RelabelingStrategy {
 public:
   InstrumentalRelabelingStrategy();
 
   InstrumentalRelabelingStrategy(double reduced_form_weight);
 
-  std::unordered_map<size_t, double> relabel(
+  bool relabel(
       const std::vector<size_t>& samples,
-      const Data* data);
+      const Data& data,
+      std::vector<double>& responses_by_sample) const;
 
   DISALLOW_COPY_AND_ASSIGN(InstrumentalRelabelingStrategy);
 
@@ -40,4 +43,6 @@ private:
   double reduced_form_weight;
 };
 
+} // namespace grf
+
 #endif //GRF_INSTRUMENTALRELABELINGSTRATEGY_H
