@@ -22,23 +22,27 @@
 #include "commons/Data.h"
 #include "splitting/SplittingRule.h"
 
-class InstrumentalSplittingRule: public SplittingRule {
+namespace grf {
+
+class InstrumentalSplittingRule final: public SplittingRule {
 public:
-  InstrumentalSplittingRule(const Data* data,
+  InstrumentalSplittingRule(size_t max_num_unique_values,
                             uint min_node_size,
                             double alpha,
                             double imbalance_penalty);
   ~InstrumentalSplittingRule();
 
-  bool find_best_split(size_t node,
+  bool find_best_split(const Data& data,
+                       size_t node,
                        const std::vector<size_t>& possible_split_vars,
-                       const std::unordered_map<size_t, double>& labels_by_sample,
+                       const std::vector<double>& responses_by_sample,
                        const std::vector<std::vector<size_t>>& samples,
                        std::vector<size_t>& split_vars,
                        std::vector<double>& split_values);
 
 private:
-  void find_best_split_value_small_q(size_t node,
+  void find_best_split_value_small_q(const Data& data,
+                                     size_t node,
                                      size_t var,
                                      size_t num_samples,
                                      double sum_node,
@@ -50,9 +54,10 @@ private:
                                      double& best_value,
                                      size_t& best_var,
                                      double& best_decrease,
-                                     const std::unordered_map<size_t, double>& responses_by_sample,
+                                     const std::vector<double>& responses_by_sample,
                                      const std::vector<std::vector<size_t>>& samples);
-  void find_best_split_value_large_q(size_t node,
+  void find_best_split_value_large_q(const Data& data,
+                                     size_t node,
                                      size_t var,
                                      size_t num_samples,
                                      double sum_node,
@@ -64,11 +69,9 @@ private:
                                      double& best_value,
                                      size_t& best_var,
                                      double& best_decrease,
-                                     const std::unordered_map<size_t, double>& responses_by_sample,
+                                     const std::vector<double>& responses_by_sample,
                                      const std::vector<std::vector<size_t>>& samples);
 
-  const Data* data;
-
   size_t* counter;
   double* sums;
   size_t* num_small_z;
@@ -82,5 +85,6 @@ private:
   DISALLOW_COPY_AND_ASSIGN(InstrumentalSplittingRule);
 };
 
+} // namespace grf
 
 #endif //GRF_INSTRUMENTALSPLITTINGRULE_H
