@@ -89,7 +89,7 @@ std::unique_ptr<Tree> TreeTrainer::train(const Data& data,
       split_vars, split_values, drawn_samples, PredictionValues()));
 
   if (!new_leaf_samples.empty()) {
-    repopulate_leaf_nodes(tree, data, new_leaf_samples, options.get_prune_empty_leaves());
+    repopulate_leaf_nodes(tree, data, new_leaf_samples, options.get_honesty_prune_leaves());
   }
 
   PredictionValues prediction_values;
@@ -104,7 +104,7 @@ std::unique_ptr<Tree> TreeTrainer::train(const Data& data,
 void TreeTrainer::repopulate_leaf_nodes(const std::unique_ptr<Tree>& tree,
                                         const Data& data,
                                         const std::vector<size_t>& leaf_samples,
-                                        const bool prune_empty_leaves) const {
+                                        const bool honesty_prune_leaves) const {
   size_t num_nodes = tree->get_leaf_samples().size();
   std::vector<std::vector<size_t>> new_leaf_nodes(num_nodes);
 
@@ -115,8 +115,8 @@ void TreeTrainer::repopulate_leaf_nodes(const std::unique_ptr<Tree>& tree,
     new_leaf_nodes[leaf_node].push_back(sample);
   }
   tree->set_leaf_samples(new_leaf_nodes);
-  if (prune_empty_leaves) {
-    tree->prune_empty_leaves();
+  if (honesty_prune_leaves) {
+    tree->honesty_prune_leaves();
   }
 }
 
@@ -203,7 +203,6 @@ bool TreeTrainer::split_node_internal(size_t node,
     return true;
   }
 
-  std::fill(responses_by_sample.begin(), responses_by_sample.end(), 0.0);
   bool stop = relabeling_strategy->relabel(samples[node], data, responses_by_sample);
 
   if (stop || splitting_rule->find_best_split(data,
